/*
 * Copyright (c) 2004, 2005, 2006, 2007, 2008
 *      Luiz Souza <loos.br@gmail.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

static const char rcsid[] = "$Id: read_address.c,v 1.1 2008/01/23 15:30:53 loos-br Exp $";

#include <unistd.h>

#include "string.h"
#include "syslog.h"
#include "vchkuser.h"

int
valid_user_char(unsigned char ch) {

    /* from vpopmail - i have nothing with this */
    if (ch == 33 || ch == 35   ||
	ch == 36 || ch == 38   ||
	ch == 39 || ch == 42   ||
	ch == 43 || ch == 35   ||
	(ch >= 45 && ch <= 57) ||
	ch == 61 || ch == 63   ||
	(ch >= 65 && ch <= 90) ||
	(ch >= 94 && ch <= 126))
	return(OK);

    return(FAIL);
}

int
read_user(string *user, unsigned char ch) {

    if (valid_user_char(ch) == FAIL) {
	vlog("bad rcpt - invalid char for user");
	return(BOUNCE);
    }
    if (ch >= 65 && ch <= 90)
	ch += 32;
    if (str_cat(user, &ch, 1) == 0) {
	vlog("fail - no memory");
	return(FAIL);
    }
    return(OK);
}

int
valid_domain_char(unsigned char ch) {

    /* from vpopmail - i have nothing with this */
    if (ch == 45 || ch == 46   ||
	(ch >= 48 && ch <= 57) ||
	(ch >= 65 && ch <= 90) ||
	(ch >= 97 && ch <= 122))
	return(OK);

    return(FAIL);
}

int
read_domain(string *domain, unsigned char ch) {

    if (valid_domain_char(ch) == FAIL) {
	vlog("bad rcpt - invalid char for domain");
	return(BOUNCE);
    }
    if (ch >= 65 && ch <= 90)
	ch += 32;
    if (str_cat(domain, &ch, 1) == 0) {
	vlog("fail - no memory");
	return(FAIL);
    }
    return(OK);
}

int
read_address(string *user, string *domain, char *env) {
 char ch;
 char *rcpt_to;
 int is_user = 1;

    /* check for null rcpt - not set probably */
    rcpt_to = getenv("REMOTERCPT");
    if (rcpt_to == NULL) {
	vlog("abort - cannot read the message rcpt to address");
	return(FAIL);
    }

    for(;;) {

	ch = *rcpt_to++;

	/* check for eof */
	if (ch == 0)
	    break;

	/* check for a domain */
	if (ch == '@') {
	    is_user = 0;
	    continue;
	}

	if (is_user) {
	    switch (read_user(user, ch)) {
		case FAIL:
		    return(FAIL);
		case BOUNCE:
		    return(BOUNCE);
	    }
	} else {
	    switch (read_domain(domain, ch)) {
		case FAIL:
		    return(FAIL);
		case BOUNCE:
		    return(BOUNCE);
	    }
	}
    }
    return(NOTBOUNCE);
}
