/*
 * Copyright (c) 2004, 2005, 2006, 2007, 2008
 *      Luiz Souza <loos.br@gmail.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

static const char rcsid[] = "$Id: syslog.c,v 1.1 2008/01/23 15:30:53 loos-br Exp $";

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>

#include "string.h"

char	*sender;
char 	*rcpt_to;
char	*sender_addr;
char	info[2048];

void
vlog_init(void) {

    /* get external info - getenv may return NULL */
    sender = getenv("REMOTEMF");
    rcpt_to = getenv("REMOTERCPT");
    sender_addr = getenv("TCPREMOTEIP");

    /* prepare log info */
    openlog("vchkuser", LOG_PID, LOG_MAIL);
    snprintf(info, sizeof(info), "(%s) <%s> -> <%s>: ",
		   sender_addr ? sender_addr : "no address",
		   sender ? sender : "RFC-821_Null_Sender",
		   rcpt_to ? rcpt_to : "no rcpt");
}

void
vlog(const char *fmt, ...) {
 va_list ap;

    /* check for fmt */
    if (fmt == NULL || strlen(fmt) == 0) return;

    /* prepend info */
    strlcat(info, fmt, sizeof(info));

    /* log it ! */
    va_start(ap, fmt);
    vsyslog(LOG_INFO, info, ap);
    va_end(ap);
}
