/*
 * Copyright (c) 2004, 2005, 2006, 2007, 2008
 *      Luiz Souza <loos.br@gmail.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

static const char rcsid[] = "$Id: vbounce.c,v 1.1 2008/01/23 15:30:54 loos-br Exp $";

#include <stdio.h>
#include <string.h>

#include "string.h"
#include "syslog.h"
#include "vchkuser.h"

int
vbounce(string *domain_path) {
 int		bounce = BOUNCE;
 string		dotqmail;
 char		buf[BUF];
 FILE		*fp;

    /* zero dotqmail */
    str_zero(&dotqmail);

    /* prepare .qmail-default file path */
    if (str_copy(&dotqmail, domain_path->s, domain_path->len) == 0 ||
	str_cat(&dotqmail, (unsigned char *)"/.qmail-default", 15) == 0)
	exit(51);

    /* open .qmail-default */
    if ((fp = fopen((char *)dotqmail.s, "r")) == NULL) {
	vlog("abort - could not open %s/.qmail-default", domain_path->s);
	return(FAIL);
    }

    /* read .qmail-default */
    while (fgets(buf, sizeof(buf), fp)) {

	/* skip comments and blank lines */
	if (*buf == '#' || *buf == '\n' || *buf == '\r')
	    continue;

	/* look for vdelivermail */
	if (strstr(buf, "vdelivermail") != NULL) {

	    /* check if the default action for vdelivermail is bounce */
	    if (strstr(buf, "bounce-no-mailbox") == NULL &&
		strstr(buf, "delete") == NULL) {

		/* default action is not bounce or delete */
		vlog("address bypass - default action for domain is not bounce");

		bounce = NOTBOUNCE;
	    }

	    /* 
	     * found vdelivermail, we are not interested in other instructions
	     */
	    break;
	}
    }
    (void)fclose(fp);
    str_free(&dotqmail);

    return(bounce);
}
