/*
 * Copyright (c) 2004, 2005, 2006, 2007, 2008
 *      Luiz Souza <loos.br@gmail.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

static const char rcsid[] = "$Id: vchkuser.c,v 1.1 2008/01/23 15:30:54 loos-br Exp $";

#include <stdlib.h>

#include "string.h"
#include "syslog.h"
#include "valias.h"
#include "vuser.h"
#include "vbounce.h"
#include "vdomain.h"
#include "vchkuser.h"
#include "read_address.h"


int
main(void) {
 string	user;
 string	domain;
 string	domain_path;

    /* init strings - safe start */
    str_zero(&user);
    str_zero(&domain);
    str_zero(&domain_path);

    /* prepare logging */
    vlog_init();

    /*
     * these tests are mandatory, so any BOUNCE return should exit 
     * immediately
     *
     * functions fails could be catched other way, but as default
     * we just let the job to vpopmail (to not block valid address)
     *
     * if we see a NOTBOUNCE the address must continue
     */

    /* read e-mail from environment variable "REMOTERCPT" */
    switch (read_address(&user, &domain)) {
	case BOUNCE:
	    exit(BOUNCE);

	case FAIL:
	    exit(NOTBOUNCE);
    }

    /*
     * no domain - deny
     */
    if (domain.len == 0) {
	vlog("bad rcpt - no domain");
	exit(BOUNCE);
    }

    /*
     * check if domain exist
     */
    switch (vdomain(&domain_path, &domain)) {
	case BOUNCE:
	    exit(BOUNCE);

	case FAIL:
	    exit(NOTBOUNCE);
    }

    /*
     * now we just exit if the function fail (see above) or if function
     * return NOTBOUNCE.
     *
     * if function return BOUNCE, the information isnt found by function
     * (but the next function may found - so let it decide)
     *
     * the last call should be BOUNCE by a direct exit call, so if any
     * of these functions trap before end, the address will be refused.
     */

    /*
     * this function checks if this domain return bounce messages
     * if not, exit here and let vpopmail deal with it
     */
    switch (vbounce(&domain_path)) {
	case FAIL:		/* fallthrought */
	case NOTBOUNCE:
	    /* do not proceed with tests, the default action is not bounce */
	    exit(NOTBOUNCE);
    }

    /*
     * if the user is a real vpopmail user, exit here and accept the
     * address
     */
    switch (vuser(&user, &domain)) {
	case FAIL:		/* fallthrought */
	case NOTBOUNCE:
	    exit(NOTBOUNCE);
    }

    /*
     * if address is an alias, exit here and accept the address
     */
    switch (valias(&user, &domain_path)) {
	case FAIL:		/* fallthrought */
	case NOTBOUNCE:
	    exit(NOTBOUNCE);
    }

    /* if no one has stopped us until this moment, goodbye address */
    vlog("bad rcpt - address refused, no more tests");
    exit(BOUNCE);
}
